@@ -21,7 +21,7 @@ unless (defined $Test::Deep::NoTest::NoTest)
 
 our ($Stack, %Compared, $CompareCache, %WrapCache, $Shallow);
 
-our $VERSION = '1.121_001';
+our $VERSION = '1.125_001';
 $VERSION =~ tr/_//d;
 
 require Exporter;
@@ -30,6 +30,8 @@ our @ISA = qw( Exporter );
 our $Snobby = 1; # should we compare classes?
 our $Expects = 0; # are we comparing got vs expect or expect vs expect
 
+our $LeafWrapper; # to wrap simple values in a test; if not set, shallow()
+
 our $DNE = \"";
 our $DNE_ADDR = Scalar::Util::refaddr($DNE);
 
@@ -89,7 +91,7 @@ while (my ($pkg, $name) = splice @constructors, 0, 2)
 }
 
 {
-  our @EXPORT_OK = qw( descend render_stack class_base cmp_details deep_diag );
+  our @EXPORT_OK = qw( descend render_stack cmp_details deep_diag );
 
   our %EXPORT_TAGS;
   $EXPORT_TAGS{preload} = [];
@@ -411,15 +413,24 @@ sub wrap
 {
   my $data = shift;
 
-  return $data if Scalar::Util::blessed($data) and $data->isa("Test::Deep::Cmp");
+  my $class = Scalar::Util::blessed($data);
+  return $data if defined $class and $data->isa("Test::Deep::Cmp");
 
-  my ($class, $base) = class_base($data);
+  if (defined $class and $data->can('as_test_deep_cmp')) {
+    my $cmp = $data->as_test_deep_cmp;
+    return $cmp if $cmp->isa('Test::Deep::Cmp');
+    Carp::confess("object in expected structure provides as_test_deep_cmp but it did not return a Test::Deep::Cmp");
+  }
+
+  my $reftype = _td_reftype($data);
 
   my $cmp;
 
-  if($base eq '')
+  if($reftype eq '')
   {
-    $cmp = shallow($data);
+    $cmp = $Test::Deep::LeafWrapper
+         ? $Test::Deep::LeafWrapper->($data)
+         : shallow($data);
   }
   else
   {
@@ -427,25 +438,27 @@ sub wrap
 
     return $WrapCache{$addr} if $WrapCache{$addr};
 
-    if($base eq 'ARRAY')
+    if($reftype eq 'ARRAY')
     {
       $cmp = array($data);
     }
-    elsif($base eq 'HASH')
+    elsif($reftype eq 'HASH')
     {
       $cmp = hash($data);
     }
-    elsif($base eq 'SCALAR' or $base eq 'REF')
+    elsif($reftype eq 'SCALAR' or $reftype eq 'REF')
     {
       $cmp = scalref($data);
     }
-    elsif(($base eq 'Regexp') or ($base eq 'REGEXP'))
+    elsif(($reftype eq 'Regexp') or ($reftype eq 'REGEXP'))
     {
       $cmp = regexpref($data);
     }
     else
     {
-      $cmp = shallow($data);
+      $cmp = $Test::Deep::LeafWrapper
+           ? $Test::Deep::LeafWrapper->($data)
+           : shallow($data);
     }
 
     $WrapCache{$addr} = $cmp;
@@ -453,29 +466,24 @@ sub wrap
   return $cmp;
 }
 
-sub class_base
+sub _td_reftype
 {
   my $val = shift;
 
-  if (ref $val)
-  {
-    my $blessed = Scalar::Util::blessed($val);
-    $blessed = defined($blessed) ? $blessed : "";
-    my $reftype = Scalar::Util::reftype($val);
+  my $reftype = Scalar::Util::reftype($val);
+  return '' unless defined $reftype;
 
+  return $reftype unless $Test::Deep::RegexpVersion::OldStyle;
 
-    if ($Test::Deep::RegexpVersion::OldStyle) {
-      if ($blessed eq "Regexp" and $reftype eq "SCALAR")
-      {
-        $reftype = "Regexp"
-      }
-    }
-    return ($blessed, $reftype);
-  }
-  else
+  my $blessed = Scalar::Util::blessed($val);
+  return $reftype unless defined $blessed;
+
+  if ($blessed && $blessed eq "Regexp" and $reftype eq "SCALAR")
   {
-    return ("", "");
+    $reftype = "Regexp"
   }
+
+  return $reftype;
 }
 
 sub render_stack
@@ -1494,9 +1502,9 @@ All C<bag> functions return an object which can have additional items added to
 it:
 
   my $bag = bag( 1, 2 );
-  $bag->add(1, 3, 1 );  # Set is now ( 1, 1, 1, 2, 3 )
+  $bag->add(1, 3, 1 );  # Bag is now ( 1, 1, 1, 2, 3 )
 
-Special care must be taken when using special comparisons within sets. See
+Special care must be taken when using special comparisons within bags. See
 L</SPECIAL CARE WITH SPECIAL COMPARISONS IN SETS AND BAGS> for details.
 
 =head3 bag
@@ -1532,10 +1540,10 @@ But the following are failures:
 
   cmp_deeply( \@got, superbagof( @elements ) );
 
-This function works much like L<< C<bag>|/bag >>, and performs a set comparison
+This function works much like L<< C<bag>|/bag >>, and performs a bag comparison
 of C<$got_v> with the elements of C<@elements>.
 
-C<supersetof> is however slightly relaxed, such that C<$got> may contain things
+C<superbagof> is however slightly relaxed, such that C<$got> may contain things
 not in C<@elements>, but must at least contain all C<@elements>.
 
 So:
@@ -1550,7 +1558,7 @@ So:
 
   cmp_deeply( \@got, subbagof(@elements) );
 
-This function works much like L<< C<bag>|/bag >>, and performs a set comparison
+This function works much like L<< C<bag>|/bag >>, and performs a bag comparison
 of C<$got_v> with the elements of C<@elements>.
 
 This is the inverse of C<superbagof>, and expects all elements in C<$got> to
